/********************************************************************\
 * Mu (Miscellaneous Utilities) is a general convenience library.   *
 * Copyright (C) 2019  Asher Gordon <AsDaGo@posteo.net>             *
 *                                                                  *
 * This file is part of Mu.                                         *
 *                                                                  *
 * Mu is free software: you can redistribute it and/or modify it    *
 * under the terms of the GNU General Public License as published   *
 * by the Free Software Foundation, either version 3 of the         *
 * License, or (at your option) any later version.                  *
 *                                                                  *
 * Mu is distributed in the hope that it will be useful, but        *
 * WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public        *
 * License along with Mu.  If not, see                              *
 * <https://www.gnu.org/licenses/>.                                 *
\********************************************************************/

/*
 * io.c -- open and close files, handling "-" or stdin/stdout
 * specially
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "io.h"

FILE *file_open(const char *filename, const char *mode) {
  char c;

  if (strcmp(filename, "-"))
    return fopen(filename, mode);

  /* Handle stdin/stdout. */
  c = *(mode++);
  if (*mode == 'b')
    mode++;
  if (*mode) {
    /* Even if it's '+', it's still not valid for stdin/stdout. */
    errno = EINVAL;
    return NULL;
  }

  switch (c) {
  case 'r':
    return stdin;

  case 'w':
  case 'a':
    return stdout;

  default:
    errno = EINVAL;
    return NULL;
  }
}

int file_close(FILE *file) {
  if (file == stdin || file == stdout)
    return 0;
  return fclose(file);
}
