/********************************************************************\
 * Mu (Miscellaneous Utilities) is a general convenience library.   *
 * Copyright (C) 2019  Asher Gordon <AsDaGo@posteo.net>             *
 *                                                                  *
 * This file is part of Mu.                                         *
 *                                                                  *
 * Mu is free software: you can redistribute it and/or modify it    *
 * under the terms of the GNU General Public License as published   *
 * by the Free Software Foundation, either version 3 of the         *
 * License, or (at your option) any later version.                  *
 *                                                                  *
 * Mu is distributed in the hope that it will be useful, but        *
 * WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public        *
 * License along with Mu.  If not, see                              *
 * <https://www.gnu.org/licenses/>.                                 *
\********************************************************************/

/*
 * safe.h -- versions of standard functions that do not return on
 * error
 */

#ifndef _SAFE_H
#define _SAFE_H

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define _SAFE_TR_NAME(name) _##name##_line
#define _SAFE_DECL(name, ...)					\
  _SAFE_TR_NAME(name)						\
    (const char *file, int line, const char *func, __VA_ARGS__)
#define _SAFE_DEF(name, ...)					\
  _SAFE_TR_NAME(name)(__FILE__, __LINE__, __func__, __VA_ARGS__)

#define xmalloc(size)		_SAFE_DEF(xmalloc, size)
#define xcalloc(nmemb, size)	_SAFE_DEF(xcalloc, nmemb, size)
#define xstrdup(str)		_SAFE_DEF(xstrdup, str)
/* Format is part of the '...' so there is always at least one
   __VA_ARGS__ for a valid call of xasprintf(). */
#define xasprintf(strp, ...)	_SAFE_DEF(xasprintf, strp, __VA_ARGS__)
#define xvasprintf(strp, fmt, ap) _SAFE_DEF(xvasprintf, strp, fmt, ap)
#define xformat(stream, width, col, indent, subindent, ...)		\
  _SAFE_DEF(xformat, stream, width, col, indent, subindent, __VA_ARGS__)
#define xformat_string(width, col, indent, subindent, ...)		\
  _SAFE_DEF(xformat, width, col, indent, subindent, __VA_ARGS__)
#define xvformat(stream, width, col, indent, subindent, fmt, ap)	\
  _SAFE_DEF(xvformat, stream, width, col, indent, subindent, fmt, ap)
#define xvformat_string(width, col, indent, subindent, fmt, ap)		\
  _SAFE_DEF(xvformat_string, width, col, indent, subindent, fmt, ap)

void *_SAFE_DECL(xmalloc, size_t);
void *_SAFE_DECL(xcalloc, size_t, size_t);
char *_SAFE_DECL(xstrdup, const char *);
unsigned _SAFE_DECL(xasprintf, char **, const char *, ...);
unsigned _SAFE_DECL(xvasprintf, char **, const char *, va_list);
void _SAFE_DECL(xformat,
		FILE *, unsigned short, unsigned short *, unsigned short,
		unsigned short, const char *, ...);
char *_SAFE_DECL(xformat_string,
		 unsigned short, unsigned short *, unsigned short,
		 unsigned short, const char *, ...);
void _SAFE_DECL(xvformat,
		FILE *, unsigned short, unsigned short *, unsigned short,
		unsigned short, const char *, va_list);
char *_SAFE_DECL(xvformat_string,
		 unsigned short, unsigned short *, unsigned short,
		 unsigned short, const char *, va_list);

#endif /* !_SAFE_H */
