/********************************************************************\
 * Mu (Miscellaneous Utilities) is a general convenience library.   *
 * Copyright (C) 2019  Asher Gordon <AsDaGo@posteo.net>             *
 *                                                                  *
 * This file is part of Mu.                                         *
 *                                                                  *
 * Mu is free software: you can redistribute it and/or modify it    *
 * under the terms of the GNU General Public License as published   *
 * by the Free Software Foundation, either version 3 of the         *
 * License, or (at your option) any later version.                  *
 *                                                                  *
 * Mu is distributed in the hope that it will be useful, but        *
 * WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public        *
 * License along with Mu.  If not, see                              *
 * <https://www.gnu.org/licenses/>.                                 *
\********************************************************************/

/*
 * test.c -- a test for Mu
 */

#include <stdio.h>

#include "options.h"

char *prog_name;

int main(int argc, char **argv) {
  int found_short_arg, found_long_arg, found_both;
  const char *short_arg, *long_arg;

  OPTION options[] = {
    {
     .short_opt	= 's',
     .has_arg	= OPT_REQUIRED,
     .type	= OPT_FILE,
     .found_arg = &found_short_arg,
     .argstr	= &short_arg,
     .file_mode	= "r",
     .arg_help	= "FILE",
     .help	= "a short option; FILE must exist and must be readable"
    },
    {
     .long_opt	= "long",
     .has_arg	= OPT_OPTIONAL,
     .type	= OPT_STRING,
     .found_arg	= &found_long_arg,
     .arg	= &long_arg,
     .help	=
     "a long option taking any string or nothing; this help text is very "
     "very very very very very loooooooooooong... seriously "
     "loooooooooooooong; I mean it (and boring too); 01234567890123456789"
     "012345678901234567890123456789012345678901234567890123456789"
     "012345678901234567890123456789012345678901234567890123456789"
     "\tTAB\tTAB\tTAB"
    },
    {
     .short_opt	= 'b',
     .long_opt	= "both",
     .has_arg	= OPT_NONE,
     .found_opt	= &found_both
    },
    {
     .short_opt	= 'r',
     .long_opt	= "required",
     .has_arg	= OPT_REQUIRED,
     .type	= OPT_STRING,
     .help	=
     "this option requires an argument and has both a long and short "
     "equivalent"
    },
    { 0 }
  };

  prog_name = argv[0];

  if (parse_opts(&argc, argv,
		 "[OPTION]... [ARG]...", "Parse options and stuff.", "Stuff.",
		 options, OPT_PERMUTE | OPT_BUNDLE | OPT_HELP))
    return 1;

  fputs("Positional arguments: {", stdout);
  for (int i = 1; i < argc; i++) {
    if (i > 1)
      fputs(", ", stdout);
    printf("\"%s\"", argv[i]);
  }
  puts("}");

  if (found_short_arg)
    printf("I found an argument to a short option: %s\n", short_arg);
  if (found_long_arg)
    printf("I found an argument to a long option: %s\n", long_arg);
  if (found_both)
    printf("I found an option which can either be long or short.\n");

  return 0;
}
